@@ -1,26 +1,26 @@
-#!/usr/bin/env python
-# Part of TotalDepth: Petrophysical data processing and presentation
-# Copyright (C) 1999-2011 Paul Ross
-# 
+#!/usr/bin/env python3
+# Part of TotalDepth: Petrophysical data processing and presentation.
+# Copyright (C) 2011-2021 Paul Ross
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-# 
+#
 # Paul Ross: apaulross@gmail.com
 """TotalDepth.LAS: Processing of LAS files.
 """
 
-__all__ = ['core', ]
+__all__ = ['core', 'ExceptionTotalDepthLAS']
 
 VERSION = (0, 1, 0)
 
@@ -31,6 +31,7 @@ RELEASE_NOTES = [
 
 from TotalDepth import ExceptionTotalDepth
 
+
 class ExceptionTotalDepthLAS(ExceptionTotalDepth):
     """Simple specialisation of an exception class for TotalDepth.LAS."""
     pass
