@@ -1,21 +1,21 @@
-#!/usr/bin/env python
-# Part of TotalDepth: Petrophysical data processing and presentation
-# Copyright (C) 1999-2011 Paul Ross
-# 
+#!/usr/bin/env python3
+# Part of TotalDepth: Petrophysical data processing and presentation.
+# Copyright (C) 2011-2021 Paul Ross
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-# 
+#
 # Paul Ross: apaulross@gmail.com
 """
 Created on May 23, 2011
@@ -225,7 +225,7 @@ class PlotLogInfo(object):
 
     def _writeIndexTableRows(self, theS, theTrie, theFilePath):
         # Write the rowspan/colspan data
-        for anEvent in theTrie.genColRowEvents():
+        for anEvent in theTrie.gen_row_column_events():
             if anEvent == theTrie.ROW_OPEN:
                 # Write out the '<tr>' element
                 theS.startElement('tr', {})
@@ -614,7 +614,7 @@ Generates plot(s) from input LIS file or directory to an output destination."""
     optParser.add_option("-x", "--xml", action="append", dest="LgFormat", default=[],
                       help="Add an XML LgFormat to use for plotting. Value is the UniqueId. Use -x? to see what LgFormats are available. [default: %default]")
     opts, args = optParser.parse_args()
-    clkStart = time.clock()
+    clkStart = time.perf_counter()
     timStart = time.time()
     # Initialise logging etc.
     logging.basicConfig(level=opts.loglevel,
@@ -629,7 +629,7 @@ Generates plot(s) from input LIS file or directory to an output destination."""
         print(myFg.longStr())
         while '?' in opts.LgFormat:
             opts.LgFormat.remove('?')
-        return 1
+        return 0
     if len(args) != 2:
         optParser.print_help()
         optParser.error("I can't do much without an input path to LIS file(s) and an output path.")
@@ -641,7 +641,7 @@ Generates plot(s) from input LIS file or directory to an output destination."""
         myResult = plotLogPassesMP(args[0], args[1], None, opts.recursive, opts.keepGoing, opts.LgFormat, opts.apiHeader, opts.jobs)
     myResult.writeHTML(os.path.join(args[1], 'index.html'), args[0])
     print('plotLogInfo', str(myResult))
-    print('  CPU time = %8.3f (S)' % (time.clock() - clkStart))
+    print('  CPU time = %8.3f (S)' % (time.perf_counter() - clkStart))
     print('Exec. time = %8.3f (S)' % (time.time() - timStart))
     print('Bye, bye!')
     return 0
