@@ -1,3 +1,22 @@
+#!/usr/bin/env python3
+# Part of TotalDepth: Petrophysical data processing and presentation.
+# Copyright (C) 2011-2021 Paul Ross
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Paul Ross: apaulross@gmail.com
 """
 Implements the Explicitly Formatted Logical Record (EFFLR) [RP66V1 Section 3 Logical Record Syntax]
 
@@ -64,8 +83,7 @@ class ExceptionEFLRObjectDuplicateLabel(ExceptionEFLRObject):
 class Set:
     """Class that represents a component set. See [RP66V1 3.2.2.1 Component Descriptor]"""
     def __init__(self, ld: LogicalData):
-        if ld.index != 0:
-            raise ExceptionEFLRSet(f'Trying to create a Set where the LogicalData {ld} index is non-zero.')
+        ld_index = ld.index
         component_descriptor = ComponentDescriptor(ld.read())
         if not component_descriptor.is_set_group:
             raise ExceptionEFLRSet(f'Component Descriptor does not represent a set but a {component_descriptor.type}.')
@@ -73,7 +91,7 @@ class Set:
         self.name: bytes = ComponentDescriptor.CHARACTERISTICS_AND_COMPONENT_FORMAT_SET_MAP['N'].global_default
         if component_descriptor.has_set_N:
             self.name = RepCode.IDENT(ld)
-        self.logical_data_consumed = ld.index
+        self.logical_data_consumed = ld.index - ld_index
 
     def __str__(self) -> str:
         """String representation."""
@@ -178,10 +196,15 @@ class Attribute(AttributeBase):
 
 class Template:
     """Class that represents a component template. See [RP66V1 3.2.2.1 Component Descriptor]"""
-    def __init__(self, ld: LogicalData):
-        """Populate the template with the Logical Data."""
+    def __init__(self):
         self.attrs: typing.List[TemplateAttribute] = []
         self.attr_label_map: typing.Dict[bytes, int] = {}
+        self.logical_data_consumed = 0
+
+
+    def read(self, ld: LogicalData):
+        """Populate the template with the Logical Data."""
+        ld_index = ld.index
         while True:
             component_descriptor = ComponentDescriptor(ld.read())
             if not component_descriptor.is_attribute_group:
@@ -382,9 +405,9 @@ class ExplicitlyFormattedLogicalRecord:
 
     def __init__(self, lr_type: int, ld: LogicalData):
         self.lr_type: int = lr_type
-        self.logical_data_consumed = 0
+        ld.rewind()
         self.set: Set = Set(ld)
-        self.template: Template = Template(ld)
+        self.template: Template = Template()
         # This object list contains all objects not including duplicates.
         self.objects: typing.List[Object] = []
         # This is the final object name map after de-duplication depending on the de-duplication strategy.
@@ -393,13 +416,15 @@ class ExplicitlyFormattedLogicalRecord:
         self.object_name_map: typing.Dict[RepCode.ObjectName, int] = {}
         temp_object_name_map: typing.Dict[RepCode.ObjectName, int] = {}
         dupes_to_remove: typing.List[int] = []
-        while ld:
-            obj = Object(ld, self.template)
-            if obj.name not in temp_object_name_map:
-                temp_object_name_map[obj.name] = len(self.objects)
-                self.objects.append(obj)
-            else:
-                self._handle_duplicate_object(obj, temp_object_name_map, dupes_to_remove)
+        if ld:
+            self.template.read(ld)
+            while ld:
+                obj = Object(ld, self.template)
+                if obj.name not in temp_object_name_map:
+                    temp_object_name_map[obj.name] = len(self.objects)
+                    self.objects.append(obj)
+                else:
+                    self._handle_duplicate_object(obj, temp_object_name_map, dupes_to_remove)
         # Clear out any duplicates then index those remaining.
         dupes_to_remove.sort()
         for i in reversed(dupes_to_remove):
