@@ -1,21 +1,21 @@
-#!/usr/bin/env python
-# Part of TotalDepth: Petrophysical data processing and presentation
-# Copyright (C) 1999-2011 Paul Ross
-# 
+#!/usr/bin/env python3
+# Part of TotalDepth: Petrophysical data processing and presentation.
+# Copyright (C) 2011-2021 Paul Ross
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-# 
+#
 # Paul Ross: apaulross@gmail.com
 """Represents the part of a plot configuration that, typically, can be obtained
 from a LIS FILM table.
@@ -272,7 +272,8 @@ class PhysFilmCfg(object):
     # 3. Primary track number
     # 4. None or secondary track number
     # b'2   ' and b'3   ' or variants, e.g. b'2\x00\x00\x00' and b'3\x00\x00\x00'
-    RE_TRAC = re.compile(b'^(LH|RH)*([TF])(\d|D)(\d)*\s*$')
+    RE_TRAC = re.compile(rb'^(LH|RH)*([TF])(\d|D)(\d)*\s*$')
+
     def __init__(self, theName, theTracks, theDest, theX):
         """Constructor.
         theName is a hashable.
