@@ -20,28 +20,26 @@
 """Tests the file indexer.
 """
 
-__author__  = 'Paul Ross'
-__date__    = '10 Feb 2011'
-__version__ = '0.8.0'
-__rights__  = 'Copyright (c) Paul Ross'
-
-import os
-import pickle
+import logging
 import sys
 import time
-import logging
-
-from TotalDepth.LIS.core import FileIndexer
-from TotalDepth.LIS.core import LogiRec
-from TotalDepth.LIS.core import LisGen
-
-sys.path.append(os.path.join(os.path.dirname(__file__), os.pardir, os.pardir))
-import BaseTestClasses
+# sys.path.append(os.path.join(os.path.dirname(__file__), os.pardir, os.pardir))
+# import BaseTestClasses
 ######################
 # Section: Unit tests.
 ######################
 import unittest
 
+from TotalDepth.LIS.core import FileIndexer
+from TotalDepth.LIS.core import LisGen
+from TotalDepth.LIS.core import LogiRec
+from tests.unit import BaseTestClasses
+
+__author__  = 'Paul Ross'
+__date__    = '10 Feb 2011'
+__version__ = '0.8.0'
+__rights__  = 'Copyright (c) Paul Ross'
+
 
 class TestPlotRecordSet(unittest.TestCase):
     """Tests PlotRecordSet."""
@@ -139,6 +137,7 @@ class TestPlotRecordSet(unittest.TestCase):
         self.assertFalse(myPrs.canPlotFromInternalRecords())
         self.assertFalse(myPrs.canPlotFromExternalRecords())
 
+
 class TestFileIndexerBase(BaseTestClasses.TestBaseLogPass):
     """Base class to create LIS files to index."""
     def _retFileHead(self):
@@ -354,25 +353,6 @@ class TestIndex_genPlotRecords(TestFileIndexerBase):
         print('Index pass[0].logPass.longStr():')
         print(myPasses[0].logPass.longStr())
 
-    def test_01(self):
-        """Test pickle/un-pickle with EntryBlocks."""
-        myBa = bytearray(self._retFileHead())
-        # Add a log pass
-        myLp = self._retLogPassGen()
-        # Stick the DFSR on the array
-        myBa += self.retPrS(myLp.lrBytesDFSR())
-        # Add some logical records
-        for i in range(4):
-            myBa += self.retPrS(myLp.lrBytes(i*100, 100))
-        myBa += self._retFileTail()
-        myFile, lis_index = self._retFileIndexSingleChannel()
-        # lis_index = FileIndexer.FileIndex(myFile)
-        pickled_index = pickle.dumps(lis_index)
-        # print(pickled_index)
-        new_index = pickle.loads(pickled_index)
-        # print(new_index)
-        self.assertEqual(lis_index.lrTypeS, new_index.lrTypeS)
-
 class Special(unittest.TestCase):
     """Special tests."""
     pass
